import json
import string

from django.conf import settings
from django.core.urlresolvers import reverse_lazy
from django.http import HttpResponse
from django.views.generic.base import TemplateView
from django.views.generic.edit import CreateView, UpdateView, DeleteView

from .models import Variable, Authentic, Wcs, AVAILABLE_SERVICES
from . import forms, utils


class AvailableService(object):
    def __init__(self, klass):
        self.id = klass.Extra.service_id
        self.label = klass._meta.verbose_name


class HomeView(TemplateView):
    template_name = 'environment/home.html'

    def get_context_data(self, **kwargs):
        context = super(HomeView, self).get_context_data(**kwargs)
        context['url_template'] = settings.SERVICE_URL_TEMPLATE
        context['variables'] = Variable.objects.all()
        context['available_services'] = [
                AvailableService(x) for x in AVAILABLE_SERVICES]
        context['installed_services'] = utils.get_installed_services()
        return context


class VariableCreateView(CreateView):
    model = Variable
    success_url = reverse_lazy('environment-home')


class VariableUpdateView(UpdateView):
    model = Variable
    success_url = reverse_lazy('environment-home')


class VariableDeleteView(DeleteView):
    model = Variable
    success_url = reverse_lazy('environment-home')


class ServiceCreateView(CreateView):
    success_url = reverse_lazy('environment-home')

    def get_context_data(self, **kwargs):
        context = super(ServiceCreateView, self).get_context_data(**kwargs)
        context['model_name'] = self.model._meta.verbose_name
        return context

    def get_initial(self):
        initial = super(ServiceCreateView, self).get_initial()
        initial['base_url'] = string.Template(settings.SERVICE_URL_TEMPLATE
                ).substitute({'app': self.model.Extra.service_id})
        return initial

    def get_template_names(self):
        return 'environment/service_form.html'

    def get(self, request, *args, **kwargs):
        self.service_id = kwargs.pop('service')
        return super(ServiceCreateView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.service_id = kwargs.pop('service')
        return super(ServiceCreateView, self).post(request, *args, **kwargs)

    def get_form_class(self):
        for service in AVAILABLE_SERVICES:
            if service.Extra.service_id == self.service_id:
                form_class = getattr(forms, service.__name__ + 'Form')
                self.model = form_class.Meta.model
                return form_class
        return None


class ServiceUpdateView(UpdateView):
    success_url = reverse_lazy('environment-home')

    def get_context_data(self, **kwargs):
        context = super(ServiceUpdateView, self).get_context_data(**kwargs)
        context['model_name'] = self.model._meta.verbose_name
        return context

    def get_template_names(self):
        return 'environment/service_form.html'

    def get(self, request, *args, **kwargs):
        self.service_id = kwargs.pop('service')
        self.get_form_class()
        return super(ServiceUpdateView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.service_id = kwargs.pop('service')
        self.get_form_class()
        return super(ServiceUpdateView, self).post(request, *args, **kwargs)

    def get_form_class(self):
        for service in AVAILABLE_SERVICES:
            if service.Extra.service_id == self.service_id:
                form_class = getattr(forms, service.__name__ + 'Form')
                self.model = form_class.Meta.model
                return form_class
        return None

class ServiceDeleteView(DeleteView):
    success_url = reverse_lazy('environment-home')
    template_name = 'environment/service_delete_confirm.html'
    context_object_name = 'object'

    def get_object(self):
        service_id = self.kwargs.pop('service')
        service_slug = self.kwargs.pop('slug')
        for service in AVAILABLE_SERVICES:
            if service.Extra.service_id == service_id:
                return service.objects.get(slug=service_slug)
        return None

def operational_check_view(request, service, slug, **kwargs):
    if service == 'wcs':
        object = Wcs.objects.get(slug=slug)
    elif service == 'authentic':
        object = Authentic.objects.get(slug=slug)
    object.check_operational()
    response = HttpResponse(content_type='application/json')
    json.dump({'operational': object.is_operational()}, response)
    return response


def installed_services_json_view(request, **kwargs):
    response = HttpResponse(content_type='application/json')
    json.dump({
        'services': [x.as_dict() for x in utils.get_installed_services()],
        'variables': dict(((v.name, v.json)
            for v in Variable.objects.all())),
        }, response)
    return response


def operational_services_json_view(request, **kwargs):
    response = HttpResponse(content_type='application/json')
    json.dump({
        'services': [x.as_dict() for x in utils.get_operational_services()],
        'variables': dict(((v.name, v.json)
            for v in Variable.objects.all())),
        }, response)
    return response
