import datetime
import urllib2
import json

from django.db import models
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError

from .utils import Zone


class Variable(models.Model):
    name = models.CharField(max_length=100, verbose_name=_('name'),
            unique=True)
    value = models.TextField(verbose_name=_('value'),
            blank=True,
            help_text=_('start with [ or { for a JSON document'))

    @property
    def json(self):
        if self.value and self.value[0] in '{[':
            return json.loads(self.value)
        return self.value

    def clean(self):
        if self.value and self.value[0] in '{[':
            try:
                json.loads(self.value)
            except ValueError:
                raise ValidationError('invalid JSON document')


class ServiceBase(models.Model):
    class Meta:
        abstract = True

    title = models.CharField(_('Title'), max_length=50)
    slug = models.SlugField()
    base_url = models.CharField(_('Base URL'), max_length=200)
    secret_key = models.CharField(_('Secret Key'), max_length=60)

    last_operational_check_timestamp = models.DateTimeField(null=True)
    last_operational_success_timestamp = models.DateTimeField(null=True)

    def is_operational(self):
        return (self.last_operational_success_timestamp is not None and
                self.last_operational_success_timestamp == self.last_operational_check_timestamp)

    def check_operational(self):
        once_now = now()
        self.last_operational_check_timestamp = once_now
        try:
            fd = urllib2.urlopen(self.base_url, timeout=10)
            fd.close()
            self.last_operational_success_timestamp = once_now
        except (urllib2.URLError, urllib2.HTTPError):
            pass
        self.save()

    def wants_frequent_checks(self):
        if self.last_operational_check_timestamp is None:
            return True
        two_minutes = datetime.timedelta(minutes=2)
        return (now() - self.last_operational_check_timestamp < two_minutes)

    def as_dict(self):
        as_dict = dict([(x, y) for (x, y) in self.__dict__.items() 
                        if type(y) in (int, str, unicode)])
        as_dict['service-id'] = self.Extra.service_id
        return as_dict


class Authentic(ServiceBase):
    class Meta:
        verbose_name = _('Authentic Identity Provider')
        verbose_name_plural = _('Authentic Identity Providers')

    class Extra:
        service_id = 'authentic'

    def get_admin_zones(self):
        return [
                Zone(_('User Management'), 'users', self.base_url + '/admin_users'),
                Zone(_('Role Management'), 'roles', self.base_url + '/admin_roles'),
                ]


class Wcs(ServiceBase):
    class Meta:
        verbose_name = _('w.c.s. Web Forms')
        verbose_name_plural = _('.w.c.s. Web Forms')

    class Extra:
        service_id = 'wcs'

    def get_admin_zones(self):
        return [
                Zone(self.title, 'webforms', self.base_url + '/admin/'),
                ]


AVAILABLE_SERVICES = [Authentic, Wcs]
